function [p,ym,yl,ylm] = suglms(x,y,f,def)
% SUGLMS calculates a least-mean-squares solution for the consequent
%        parameters in the Sugeno-Takagi model.
%
% [P,Ym,Yl,Ylm] = SUGLMS(X,Y,F) 
%    Input:
%       X .....	input data matrix
%	Y .....	output data vector
%       F ..... fuzzy partition matrix (membership degrees)
%       DEF ... default value returned when the sum of grades
%               equals to one (optional, defaults to 0)
%    Output:
%       P .....	consequents parameters for every cluster
%	Ym ....	global model output for the given input data
%	Yl ....	output of local submodels (corresponding to clusters)
%	Ylm ...	output of local submodels with data corresponding to
%               degrees < 0.2 masked with NaN's (for plots)
%
%    Example:
%	x = (0:0.02:1)'; y = sin(7*x);
%       f = mgrade(x',mfequ(x,2,3))';
%       [p,ym,yl,ylm] = suglms([x ones(size(x))],y,f);
%	subplot(211); plot(x,ylm,'.',x,[y ym]); title('Fitting y = sin(7*x)')
%	subplot(212); plot(x,f); title('Membership functions')

% (c) Robert Babuska, January 1994

if nargin < 4, def = 0; end;                            % no default supplied
[mx,nx] = size(x); [mf,nf] = size(f); p = zeros(nf,nx);
sumDOF = sum([zeros(1,mf);f'])'*ones(1,nf);             % sum of DOFs
NoRule = sumDOF == 0;                                   % no rule applicable
sumDOF = sumDOF + NoRule;                               % set zeros to one
x1 = zeros(mx,nf*nx); f1 = x1; 				% auxilliary variables
xx = zeros(nx,nf*mx);                   		% auxilliary variable
x = x'; xx(:) = x(:)*ones(1,nf); 
x1(:) = xx';						% reshape data matrix
f1(:) = f(:)./sumDOF(:)*ones(1,nx);			% reshape partition matrix
p(:) = f1.*x1 \ y;					% find LMS solution

if nargout > 1,						% calculate model output
  yl = x'*p';						% local models
  ym = sum([zeros(1,mx); ...
	    (yl.*f./sumDOF)'])' + def.*NoRule(:,1); 	% global model
  ylm = yl;
  mask = find(f < 0.2);					% find membership degrees < 0.2
  ylm(mask) = NaN*ones(size(mask));     		% mask them with NaN's for plots
end;

